// RoundedButton.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.
//
// We override the Button method which draws the inside of the button,
// in order to create a nice rounded-3D effect.  The background drawing
// is cached in a bitmap for efficiency.

import netscape.application.*;
import netscape.util.*;

public class RoundedButton extends Button {
    Bitmap bitmap = null;         // Stores a bitmap of the interior

    // --------------  Constructors and initialization  --------------

    public RoundedButton(int x, int y, int width, int height) {
        super(x, y, width, height);
    }

    // Create a bitmap used to draw the Button's interior
    private void createBitmap(int width, int height) {
        Graphics g;
        int i, y;
        float f, brightness;

        width++;
        height++;
        bitmap = new Bitmap(width, height);
        g = new Graphics(bitmap);
        y = 0;
        // Start at the top with a brightness of 0.8;
        brightness = 0.7f;
        // Decrease brightness as we go down by a small increment
        f = 0.5f / (float)height;
        // Fill the Rect with a series of lines of gradually
        // decreasing brightness
        while (y < height) {
            g.setColor(Color.colorForHSB(0.0f, 0.0f, brightness));
            g.drawLine(0, y, width, y);
            y++;
            brightness -= f;
        }
    }


    // -------------------  Drawing methods  ---------------------

    public void drawViewInterior(Graphics g, String title, Image image,
                                 Rect interiorRect) {
        int i;

        // Create the bitmap if necessary
        if (bitmap == null)
            createBitmap(interiorRect.width, interiorRect.height);

        // Copy the bitmap to the screen
        g.drawBitmapAt(bitmap, interiorRect.x, interiorRect.y);

        // Now draw the title
        g.setColor(titleColor());
        drawViewTitleInRect(g, title, font(), interiorRect,
                            Graphics.CENTERED);
    }


    // --------------------  Resizing methods  ---------------------

    // If the Button size changes, we have to generate a new interior bitmap
    public void setBounds(Rect rect) {
        if ((rect.width != width()) || (rect.height != height())) {
            bitmap = null;
            // The bitmap will now be regenerated in drawViewInterior
        }
        super.setBounds(rect);
    }

}
