// Bubble.java
// By Ned Etcode
// Copyright 1995, 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** This DrawingSequence handles the animation of a bubble. We always start
  * near the bottom of the range and move to the top, expanding as we go
  * up. Implementing the Movable interface adds a notion of position and
  * bounds. We override the nextFrame() method in order to adjust our
  * position for each frame. The owner (TankView) then draws us at our
  * current position.
  */
public class Bubble extends DrawingSequence implements Movable {
    Rect  tankBounds;
    Rect  currentBounds;
    Rect  lastBounds;
    Image images[];

    public Bubble() {
        super(null);

        setPlaybackMode(FORWARD_LOOP);
        setFrameRate(100);                // in milliseconds

        images = new Image[5];
        images[0] = Bitmap.bitmapNamed("bubl0.gif");
        images[1] = Bitmap.bitmapNamed("bubl1.gif");
        images[2] = Bitmap.bitmapNamed("bubl2.gif");
        images[3] = Bitmap.bitmapNamed("bubl3.gif");
        images[4] = Bitmap.bitmapNamed("bubl4.gif");

        currentBounds = new Rect(0, 0, width(), height());
        lastBounds = new Rect(currentBounds);
    }

    /** Reset the current drawing position
      */
    public void setPosition(int x, int y) {
        currentBounds.x = x;
        currentBounds.y = y;
    }

    /** Reset the current moving limits
      */
    public void setTankBounds(int x, int y, int width, int height) {
        tankBounds = new Rect(x, y, width, height);

        // We want to move a constant number of pixels per frame, so set the
        // frame count depending on the height.  We'll move 5 pixels per frame
        setFrameCount((tankBounds.height - 20) / 5);
    }

    public Rect currentBounds() {
        return currentBounds;
    }

    public Rect lastBounds() {
        return lastBounds;
    }

    public int width() {
        return images[4].width();
    }

    public int height() {
        return images[4].height();
    }

    public boolean nextFrame() {
        int frame = currentFrameNumber() + 1;

        // Save the previously drawn bounds
        lastBounds.setBounds(currentBounds);

        // Move up 5 pixels per frame and expand as we go up
        currentBounds.y = tankBounds.height - 15 - frame * 5;

        // This will change the frame number and ask our owner view to draw us
        return super.nextFrame();
    }

    public Image currentImage() {
        int i = currentFrameNumber() / 15;

        if (i > 4)
            i = 4;
        return images[i];
    }

    /** Just draw white circle in the current bounds
      */
    public void drawAt(Graphics g, int x, int y) {
        currentImage().drawCentered(g, x, y, width(), height());
    }

}
