// IconView.java
// By Ned Etcode
// Copyright 1995, 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** An abstract superclass of the views seen in the supply palette. The
  * newSequence() method allows us to request a DrawingSequence (which will
  * also be our dragImage) from instances of all the various palette views.
  */
public abstract class IconView extends View implements DragSource {
    DragSession dragSession;          // A DragSession used for drag 'N drop
    boolean isHighlighted;
    int x, y;
    Hashtable info;

    public IconView() {
        super();
    }

    public View sourceView(DragSession session) {
        return this;
    }

    public void dragWasAccepted(DragSession session) {
        // Drag is complete
        dragSession = null;
    }

    public boolean dragWasRejected(DragSession session) {
        // Drag is complete, but rejected, so return true to animate
        // image back to initial location.
        dragSession = null;
        return true;
    }

    public void drawView(Graphics g) {
        if (isHighlighted()) {
            g.setColor(Color.lightGray);
            g.drawRect(0, 0, width(), height());
        }
    }

    public boolean mouseDown(MouseEvent event) {
        x = event.x;
        y = event.y;
        return true;
    }

    /** Start a drag session.
      */
    public void mouseDragged(MouseEvent event) {
        if (dragSession != null)       // Don't start more than one session!
            return;

        dragSession = new DragSession(this, new IconImage(this),
                                      0, 0, x, y,
                                      "icon", this);
    }

    public boolean wantsMouseTrackingEvents() {
        return true;
    }

    public void mouseEntered(MouseEvent e) {
        isHighlighted = true;
        draw();
    }

    public void mouseExited(MouseEvent e) {
        isHighlighted = false;
        draw();
    }

    public boolean isHighlighted() {
        return isHighlighted;
    }

    public abstract DrawingSequence newSequence();
}
