// JavaScriptLab.java
// By Ned Etcode
// Copyright 1995, 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;
import netscape.javascript.*;

/** Simple example to test out the JavaScript to IFC connection.
  * Note that there is no Main method here. This can only run in Navigator
  * for the time being.
  */
public class JavaScriptLab extends Application implements Target {
    ScrollGroup scrollGroup;
    TextView    textView;
    TextField   textField;
    JSObject    javaScriptObject;
    Button      button;
    public int      currentHistoryIndex;
    public Vector   commandHistory;

    public final static int MAX_HISTORY_COUNT = 20;
    public final static String EXEC_JAVASCRIPT = "execJavaScript";
    public final static String NEXT_COMMAND = "next";
    public final static String PREV_COMMAND = "prev";
    public final static String COPY_LOG = "copy";

    private static Range emptyRange = new Range(0, 0);

    /** This method gets called as the applet starts up.
      * We will normally set up the view hierarchy here.
      */
    public void init() {
        super.init();

        commandHistory = new Vector();
        currentHistoryIndex = 0;
        mainRootView().setColor(Color.lightGray);

        scrollGroup = new ScrollGroup(5, 5, 310, 150);
        textView = new TextView(0, 0, 310, 1);
        textView.setEditable(false);
        scrollGroup.setContentView(textView);
        scrollGroup.setBorder(BezelBorder.loweredBezel());
        scrollGroup.setHasVertScrollBar(true);
        mainRootView().addSubview(scrollGroup);

        textField = new TextField(5, 170, 250 , 24);
        textField.setCommand(EXEC_JAVASCRIPT);
        textField.setTarget(this);
        mainRootView().addSubview(textField);

        button = new Button(258, 170, 60, 24);
        button.setTitle("Copy Log");
        button.setTarget(this);
        button.setCommand(COPY_LOG);
        mainRootView().addSubview(button);

        javaScriptObject = JSObject.getWindow(AWTCompatibility.awtApplet());

    }

    /** This method is the Target interface. It allows us to process
      * events properly.
      */
    public void performCommand(String command, Object object)   {
        if(command.equals(EXEC_JAVASCRIPT)) {
            execJavaScript();
        } else if(command.equals(NEXT_COMMAND)) {
            nextCommand();
        } else if(command.equals(PREV_COMMAND)) {
            previousCommand();
        } else if(command.equals(COPY_LOG)) {
            copyLog();
        }
    }

    /** Prepends the <b>text</b> to the top of the textView.
      */
    public void logString(String text)  {
        textView.replaceRangeWithString(emptyRange, text+"\n");
    }

    /** Adds the current command to the Vector. Makes sure
      * we don't go over the maxium number of commands.
      */
    public void addCommandToList()  {
        commandHistory.addElement(textField.stringValue());
        if(commandHistory.count() > MAX_HISTORY_COUNT)  {
            commandHistory.removeElementAt(0);
        }
        currentHistoryIndex = commandHistory.count()-1;
    }

    /** Displays the next command, if any in the command history.
      */
    public void nextCommand()   {
        currentHistoryIndex++;
        if(currentHistoryIndex >= commandHistory.count())
            currentHistoryIndex = commandHistory.count()-1;
        displayCommand();
    }

    /** Displays the previous command, if any in the command history.
      */
    public void previousCommand()   {
        currentHistoryIndex--;
        if(currentHistoryIndex < 0)
            currentHistoryIndex = 0;
        displayCommand();
    }

    /** Copies the data from the textView to the HTML object.
      * There was some trouble passing newlines this way,
      * so we convert them to semicolons.
      */
    public void copyLog()   {
        String rawString = textView.string();
        rawString = rawString.replace('\n', ';');
        javaScriptObject.eval("document.History.LogTextView.value = \"" +
                                rawString + "\"");
    }

    /** Display the command at </b>commandIndex</b> in the textField.
      */
    private void displayCommand()   {
        if(currentHistoryIndex < 0
            || currentHistoryIndex > commandHistory.count()-1
            || commandHistory.count() < 1)
            return;
        textField.setStringValue(
            (String)commandHistory.elementAt(currentHistoryIndex));
    }

    /** Executes the command in the textField. Logs the command into the
      * textView and adds the command commandHistory.
      */
    private void execJavaScript() {
        addCommandToList();
        logString(textField.stringValue());
        if(textField.stringValue() == null
            || textField.stringValue().length() < 1)    {
            textField.setStringValue("");
            textField.setInsertionPoint(1);
            return;
        }
        try {
            javaScriptObject.eval(textField.stringValue());
        } catch (Throwable e) {  /// This probably should not catch Throwable but..
            logString("error:\n    "  + e);
            if(e.getMessage() != null)
               logString("    " + e.getMessage().toString());
            e.printStackTrace();
        } finally {
            textField.setStringValue("");
            textField.setInsertionPoint(1);
        }
    }

}
