import netscape.application.*;
import netscape.util.*;

public class GridController extends InternalWindow implements Target {
    TextField buttonCount;
    TextField rowCount;
    TextField columnCount;
    Vector    flowChoice;

    static final String GEN_COMMAND = "GenGrid";

    /** This window used the PackLayout manager to create a
      * window for defining the properties of a test Grid window.
      */
    public GridController(int x, int y, int width, int height)  {
        super(x, y, width, height);

        setTitle("Grid Control");
        setResizable(true);

        contentView().setLayoutManager(new PackLayout());

        buttonCount = newTextFieldRow("# Buttons:", "16");
        rowCount = newTextFieldRow("Row count:", "4");
        columnCount = newTextFieldRow("Column count:", "4");

        Vector foo = new Vector(2);
        foo.addElement("DOWN");
        foo.addElement("ACROSS");
        flowChoice = newRadioGroup(foo, "Flow Direction");
        addNewGenerateButton();

        contentView().layoutView(0,0);
    }

    public Vector newRadioGroup(Vector labels, String titleValue)  {
        int             i,max;
        PackLayout      layoutManager;
        PackConstraints constraints;
        Button          radioButton;
        ContainerView   containerView;
        Vector          returnVector = new Vector(labels.count());

        containerView = newContainerView();
        containerView.setTitle(titleValue);
        layoutManager = (PackLayout)containerView.layoutManager();

        max = labels.count();
        for(i = 0; i < max; i++)  {
            radioButton = new Button(1, 1, 1, 1);
            radioButton.setType(Button.RADIO_TYPE);
            radioButton.setTitle((String)labels.elementAt(i));
            radioButton.setImage(Bitmap.bitmapNamed(
                        "netscape/application/RadioButtonOff.gif"));
            radioButton.setAltImage(Bitmap.bitmapNamed(
                        "netscape/application/RadioButtonOn.gif"));
            radioButton.setBordered(false);
            containerView.addSubview(radioButton);
            constraints = new PackConstraints();
            constraints.setExpand(true);
            constraints.setFillX(true);
            constraints.setFillY(true);
            constraints.setPadY(2);
            constraints.setPadX(2);
            constraints.setSide(PackConstraints.SIDE_LEFT);
            layoutManager.setConstraints(radioButton, constraints);
            if(i == 0)
                radioButton.setState(true);
            returnVector.addElement(radioButton);
        }
        return returnVector;
    }


    public TextField newTextFieldRow(String labelText, String defaultValue)    {
        PackLayout      layoutManager;
        PackConstraints constraints;
        ContainerView   containerView;
        TextField       label;
        TextField       entryTextField;

        containerView = newContainerView();
        containerView.setBorder(null);
        layoutManager = (PackLayout)containerView.layoutManager();

        label = new TextField(1, 1, 1, 1);
        label.setBorder(null);
        label.setBackgroundColor(Color.lightGray);
        label.setStringValue(labelText);
        label.setEditable(false);
        label.setSelectable(false);
        label.sizeToMinSize();
        containerView.addSubview(label);
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_LEFT);
        constraints.setPadX(2);
        layoutManager.setConstraints(label, constraints);

        entryTextField = new TextField(1, 1, 1, 1);
        entryTextField.setStringValue(defaultValue);
        entryTextField.sizeToMinSize();
        containerView.addSubview(entryTextField);
        constraints.setExpand(true);
        constraints.setFillX(true);
        layoutManager.setConstraints(entryTextField, constraints);
        return entryTextField;
    }

    public void addNewGenerateButton()  {
        PackLayout      layoutManager;
        PackConstraints constraints;
        ContainerView   containerView;
        Button          button;

        containerView = newContainerView();
        containerView.setBorder(null);
        layoutManager = (PackLayout)containerView.layoutManager();

        button = new Button(1,1,1,1);
        button.setTitle("Generate Grid");
        button.setTarget(this);
        button.setCommand(GEN_COMMAND);
        containerView.addSubview(button);
        constraints = new PackConstraints();
        constraints.setExpand(true);
        constraints.setFillX(true);
        constraints.setFillY(true);
        constraints.setPadY(5);
        constraints.setPadX(5);
        constraints.setSide(PackConstraints.SIDE_LEFT);
        layoutManager.setConstraints(button, constraints);
    }

    public ContainerView newContainerView() {
        ContainerView   containerView;
        PackLayout      layoutManager;
        PackConstraints constraints;

        containerView = new ContainerView(1, 1, 1, 1);
        containerView.setLayoutManager(new PackLayout());
        contentView().addSubview(containerView);
        layoutManager = (PackLayout)contentView().layoutManager();
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_TOP);
        constraints.setExpand(true);
        constraints.setFillX(true);
        constraints.setFillY(true);
        layoutManager.setConstraints(containerView, constraints);
        return containerView;
    }

    public void performCommand(String command, Object object)   {
        LayoutExample app;
        int flowDirection;

        app = ((LayoutExample)Application.application());

        if(((Button)flowChoice.elementAt(0)).state())
            flowDirection = GridLayout.FLOW_DOWN;
        else    {
            flowDirection = GridLayout.FLOW_ACROSS;
        }

        app.gridTest(rowCount.intValue(),
                      columnCount.intValue(),
                      buttonCount.intValue(),
                      flowDirection);
    }

}
