// LayoutExample.java
// By Ned Etcode
// Copyright 1995, 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

import GridController;

/** This example application exercises the various LayoutManager
  * implementations provided with IFC.
  */
public class LayoutExample extends Application {

    public void init() {
        super.init();

        simpleWindow();
        alertWindow("This is an alert", "Ok" , "Cancel", "What?");
        packSide();

        relativeLayout();
        packAnchor();
        gridControl();
    }

    /** A simple window using some of the PackLayout features
      */
    public void simpleWindow()  {
        InternalWindow window = new InternalWindow(40, 40, 170, 170);
        PackLayout layoutManager;
        PackConstraints constraints;
        TextField statusField;
        Button button;
        TextField textField;

        layoutManager = new PackLayout();
        window.contentView().setLayoutManager(layoutManager);

        window.setTitle("Simple");
        window.setResizable(true);
        window.setCloseable(true);

        textField = new TextField(24, 128, 64, 24);
        textField.setCommand("textWasEntered");
        window.addSubview(textField);
        constraints = new PackConstraints();
        constraints.setFillX(true);
        layoutManager.setConstraints(textField, constraints);

        button = new Button(24, 64, 64, 24);
        button.setTitle("Push Me");
        window.addSubview(button);
// We're reusing the constraints set above
        constraints = new PackConstraints();
        constraints.setFillX(true);
        constraints.setFillY(true);
        constraints.setExpand(true);
        constraints.setSide(PackConstraints.SIDE_RIGHT);
        layoutManager.setConstraints(button, constraints);

        statusField = new TextField(24, 24, 100, 24);
        statusField.setBorder(null);
        statusField.setEditable(false);
        statusField.setBackgroundColor(Color.lightGray);
        statusField.setFont(Font.fontNamed("Courier", Font.PLAIN, 12));
        statusField.setStringValue("Hello World");
        window.addSubview(statusField);

        window.contentView().layoutView(0, 0);
        window.show();
    }

    /** A simple Alert window with a textField and three buttons
      */
    public void alertWindow(String alertText, String defaultText, String secondText, String thirdText)    {
        InternalWindow window = new InternalWindow(220, 40, 170, 170);
        PackLayout layoutManager;
        PackConstraints constraints;
        Button defaultButton;
        Button secondButton;
        Button thirdButton;
        TextField textField;

        window.setTitle("Alert!");
        window.setResizable(true);
        window.setCloseable(true);

        layoutManager = new PackLayout();
        window.contentView().setLayoutManager(layoutManager);

        textField = new TextField(1, 1, 1, 1);
        textField.setJustification(Graphics.CENTERED);
        textField.setBackgroundColor(Color.lightGray);
        textField.setEditable(false);
        textField.setSelectable(false);
        textField.setStringValue("Some Alert Message Here");
        window.addSubview(textField);
        constraints = new PackConstraints();
        constraints.setFillX(true);
        constraints.setFillY(true);
        constraints.setExpand(true);
        constraints.setSide(PackConstraints.SIDE_TOP);
        constraints.setPadX(5);
        constraints.setPadY(5);
        layoutManager.setConstraints(textField, constraints);

        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_RIGHT);
        constraints.setPadX(5);
        constraints.setPadY(3);

        if(defaultText != null)  {
            defaultButton = new Button();
            defaultButton.setTitle(defaultText);
            window.addSubview(defaultButton);
            layoutManager.setConstraints(defaultButton, constraints);
        }

        if(secondText != null)  {
            secondButton = new Button();
            secondButton.setTitle(secondText);
            window.addSubview(secondButton);
            layoutManager.setConstraints(secondButton, constraints);
        }

        if(thirdText != null)    {
            thirdButton = new Button();
            thirdButton.setTitle(thirdText);
            window.addSubview(thirdButton);
            layoutManager.setConstraints(thirdButton, constraints);
        }


        window.contentView().layoutView(0, 0);
        window.show();
    }

    /** Tests all the Side Tags on PackLayout.
      */
    public void packSide()  {
        PackLayout layoutManager;
        PackConstraints constraints;
        InternalWindow window = new InternalWindow(400, 40, 170, 170);
        Button button;

        window.setResizable(true);
        window.setTitle("Testing PackLayout Side Tag");
        window.setCloseable(true);

        layoutManager = new PackLayout();
        window.contentView().setLayoutManager(layoutManager);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.SIDE_TOP);
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_TOP);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.SIDE_RIGHT);
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_RIGHT);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.SIDE_BOTTOM);
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_BOTTOM);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.SIDE_LEFT);
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_LEFT);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.SIDE_TOP);
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_TOP);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.SIDE_RIGHT);
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_RIGHT);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.SIDE_BOTTOM);
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_BOTTOM);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.SIDE_LEFT);
        constraints = new PackConstraints();
        constraints.setSide(PackConstraints.SIDE_LEFT);
        layoutManager.setConstraints( button, constraints);

        window.contentView().layoutView(0, 0);
        window.show();
    }

    /** Tests the default view resizing behaviors
      */
    public void relativeLayout()    {
        InternalWindow window = new InternalWindow(40, 220, 170, 230);
        Button button;

        window.setResizable(true);
        window.setTitle("Testing Relative Layouts");
        window.setCloseable(true);

        button = new Button(10, 10, 1, 1);
        button.setTitle("RIGHT/BOTTOM Changes");
        button.sizeToMinSize();
        window.addSubview(button);

        button = new Button(10, 30, 1, 1);
        window.addSubview(button);
        button.setTitle("WIDTH Changes");
        button.sizeToMinSize();
        button.setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);

        button = new Button(10, 50, 1, 1);
        window.addSubview(button);
        button.setTitle("LEFT/TOP Changes");
        button.sizeToMinSize();
        button.setHorizResizeInstruction(View.LEFT_MARGIN_CAN_CHANGE);
        button.setVertResizeInstruction(View.TOP_MARGIN_CAN_CHANGE);

        button = new Button(10, 100, 1, 1);
        window.addSubview(button);
        button.setTitle("HEIGHT Changes");
        button.sizeToMinSize();
        button.setVertResizeInstruction(View.HEIGHT_CAN_CHANGE);

        button = new Button(10, 150, 1, 1);
        window.addSubview(button);
        button.setTitle("SIZE Changes");
        button.sizeToMinSize();
        button.setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);
        button.setVertResizeInstruction(View.HEIGHT_CAN_CHANGE);

        window.contentView().layoutView(0, 0);
        window.show();
    }

    public void gridControl()   {
        GridController tmp;
        tmp = new GridController(400, 220, 170, 220);
        tmp.show();
    }

    /** Tests all the Anchor Tags on PackLayout.
      */
    public void packAnchor()    {
        PackLayout layoutManager;
        PackConstraints constraints;
        InternalWindow window = new InternalWindow(220, 220, 170, 200);
        Button button;

        window.setResizable(true);
        window.setTitle("Testing PackLayout Anchor Tag");
        window.setCloseable(true);

        layoutManager = new PackLayout();
        window.contentView().setLayoutManager(layoutManager);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.ANCHOR_NORTH);
        constraints = new PackConstraints();
        constraints.setAnchor(PackConstraints.ANCHOR_NORTH);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.ANCHOR_NORTHEAST);
        constraints = new PackConstraints();
        constraints.setAnchor(PackConstraints.ANCHOR_NORTHEAST);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.ANCHOR_EAST);
        constraints = new PackConstraints();
        constraints.setAnchor(PackConstraints.ANCHOR_EAST);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.ANCHOR_SOUTHEAST);
        constraints = new PackConstraints();
        constraints.setAnchor(PackConstraints.ANCHOR_SOUTHEAST);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.ANCHOR_SOUTH);
        constraints = new PackConstraints();
        constraints.setAnchor(PackConstraints.ANCHOR_SOUTH);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.ANCHOR_SOUTHWEST);
        constraints = new PackConstraints();
        constraints.setAnchor(PackConstraints.ANCHOR_SOUTHWEST);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.ANCHOR_WEST);
        constraints = new PackConstraints();
        constraints.setAnchor(PackConstraints.ANCHOR_WEST);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.ANCHOR_NORTHWEST);
        constraints = new PackConstraints();
        constraints.setAnchor(PackConstraints.ANCHOR_NORTHWEST);
        layoutManager.setConstraints( button, constraints);

        button = new Button(1, 1, 1, 1);
        window.addSubview(button);
        button.setTitle(""+PackConstraints.ANCHOR_CENTER);
        constraints = new PackConstraints();
        constraints.setAnchor(PackConstraints.ANCHOR_CENTER);
        layoutManager.setConstraints( button, constraints);

        window.contentView().layoutView(0, 0);
        window.show();
    }

    /**  Test the GridLayout class. Creates an InternalWindow with
      *  the appropriate settings. This method is called from the
      *  GridController.
      */
    public void gridTest(int rowCount, int columnCount, int numButtons, int flowDirection)  {
        GridLayout layoutManager;
        InternalWindow window = new InternalWindow(410, 230, 170, 170);
        int i;

        window.setResizable(true);
        window.setTitle("Grid Test");
        window.setCloseable(true);

        layoutManager = new GridLayout(rowCount, columnCount , 4, 4, flowDirection);
        window.contentView().setLayoutManager(layoutManager);

        for(i = 1; i <= numButtons; i++)   {
            Button defbutton = new Button();
            defbutton.setTitle("Button " + i );
            window.addSubview(defbutton);
        }
        window.contentView().layoutView(0, 0);
        window.show();
    }

    // This method allows the application to run as a stand alone application.
    public static void main(String args[]) {
        LayoutExample app;
        ExternalWindow mainWindow;
        Size size;

        app = new LayoutExample();
        mainWindow = new ExternalWindow();
        app.setMainRootView(mainWindow.rootView());
        size = mainWindow.windowSizeForContentSize(640, 480);
        mainWindow.sizeTo(size.width, size.height);
        mainWindow.show();

        app.run();
        System.exit(0);
    }

}
