// FileIcon.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** A simple View that just initiates a drag session when the mouse is
  * clicked and dragged over it, with a file name String as the drag item.
  * Displays a typical text document bitmap and a title underneath.
  */
public class FileIcon extends View implements DragSource {
    static Bitmap bitmap = null;  // The "document" bitmap
    static Font font = null;      // The file name font
    String filename;              // The file name
    Rect   nameRect;              // The file name drawing rectangle

    // --------------  Constructors and initialization  --------------

    public FileIcon(int x, int y, String file) {
        int width;
        int height;

        filename = file;

        if (bitmap == null) {
            bitmap = Bitmap.bitmapNamed("document.gif");
        }
        if (font == null) {
            font = new Font("Helvetica", Font.PLAIN, 14);
        }

        /* size the name rect and the view itself to fit the bitmap
         * and the name string
         */
        width = Math.max(font.fontMetrics().stringWidth(filename) + 4,
                         bitmap.width());
        height =  bitmap.height() + font.fontMetrics().height();
        nameRect = new Rect(0, bitmap.height(), width,
                            font.fontMetrics().height());
        setBounds(x, y, width, height);
    }

    //  ---------------  View drawing methods  ---------------------

    public void drawView(Graphics g) {
        g.drawBitmapAt(bitmap, 0, 0);
        g.setColor(Color.black);
        g.setFont(font);
        g.drawStringInRect(filename, nameRect, Graphics.CENTERED);
    }


    //  ----------------  Drag session support methods  ---------------

    public boolean mouseDown(MouseEvent event) {
        /* start sending us mousDragged events */
        return true;
    }

    /** Start a drag session.  Note that the locations must be given in terms
      * of the coordinate space of the RootView.
      */
    public void mouseDragged(MouseEvent event) {
        new DragSession(this, bitmap, 0, 0, event.x, event.y, null, filename);
    }

    public View sourceView(DragSession session) {
        return this;
    }

    public void dragWasAccepted(DragSession session) {
    }

    public boolean dragWasRejected(DragSession session) {
        /* animate back */
        return true;
    }
}

