// LightStrip.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.


import netscape.application.*;
import netscape.util.*;

/** A DrawingSequence which tiles a drawn image of a yellow circle along the
  * specified drawing rectangle.  Used by MarqueeView's border.
  */
public class LightStrip extends DrawingSequence {
    int unitWidth;  // The width and height of a single "light" in the strip
    int unitHeight;

    // --------------  Constructors and initialization  --------------

    public LightStrip(DrawingSequenceOwner owner, int width, int height) {
        super(owner);

        setFrameCount(4);
        setFrameRate(233);
        setPlaybackMode(DrawingSequence.FORWARD_LOOP);
        this.unitWidth = width;
        this.unitHeight = height;
    }

    public LightStrip(int width, int height) {
        this(null, width, height);
    }


    //  ---------------  DrawingSequence methods  -----------------

    /** Draws a single "light" at the given position. */
    public void drawAt(Graphics g, int x, int y) {
        int     w, h;

        w = unitWidth / 3;
        h = unitHeight / 3;

        g.setColor(Color.darkGray);
        g.fillRect(x, y, unitWidth, unitHeight);
        g.setColor(Color.yellow);
        g.fillOval(x + w, y + h, w, h);
    }

    /** DrawTiled fills the given rectangle with lights.  The "lit" lights
      * are determined by the current frame number.
      */
    public void drawTiled(Graphics g, int x, int y, int width, int height) {
        int     i, n, w, h, frame;
        float   xx;

        frame = currentFrameNumber();

        /* first draw the dark blue background */
        g.setColor(new Color(0, 0, 64));
        g.fillRect(x, y, width, height);

        /* now draw the lights */
        if (width > height) {
            // Horizontal mode
            n = width / unitWidth + 1;
            w = unitWidth / 3;
            h = height / 3;
            x += w;
            y += h;
            for (i = 0; i < n; i++) {
                g.setColor(((i % 4) == frame) ? Color.yellow : Color.black);
                g.fillOval(x, y, w, h);
                x += unitWidth;
            }
        } else {
            /* vertical mode */
            n = height / unitHeight + 1;
            w = width / 3;
            h = unitHeight / 3;
            x += w;
            y += h;
            for (i = 0; i < n; i++) {
                g.setColor(((i % 4) == frame) ? Color.yellow : Color.black);
                g.fillOval(x, y, w, h);
                y += unitHeight;
            }
        }
    }

    public void drawScaled(Graphics g, int x, int y, int width, int height) {
        drawTiled(g, x, y, width, height);
    }

    public int width() {
        return unitWidth;
    }

    public int height() {
        return unitHeight;
    }
}
