// MarqueeView.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** MarqueeView is a PresentationView subclass which by default uses the
  * LightStrip class for its border, and displays its text by simulating
  * a flashing light bulb grid.
  *
  * The lightbulb grid is created by overlaying a ScreenView on top of a
  * SimpleTextView.  The ScreenView actually does the animating here.
  */
public class MarqueeView extends PresentationView {
    SimpleTextView textView;      // Draws the text
    ScreenView     screenView;    // Draws a grid of lines over the text
    int            index;         // Current index into strings array

    // --------------  Constructors and initialization  --------------

    public MarqueeView(String file) {
        this(0, 0, 200, 200, file);
    }

    public MarqueeView(int x, int y, int width, int height, String file) {
        /* initialize the PresentationView, specifying a LightStrip for
         * out border
         */
        super(x, y, width, height,
              new ImageBorder(new LightStrip(20, 20), 20, 20), null);

        Rect interiorRect;

        /* our background is basic black */
        setBackgroundColor(Color.black);

        /* initialize the SimpleTextView to fit the interior rectangle */
        interiorRect = interiorRect();
        textView = new SimpleTextView(interiorRect.x, interiorRect.y,
                                  interiorRect.width, interiorRect.height);
        textView.setTextColor(Color.white); // Draw white text on our black

        /* biiiig font */
        textView.setFont(new Font("Helvetica", Font.BOLD, 54));
        textView.setHorizResizeInstruction(WIDTH_CAN_CHANGE);
        textView.setVertResizeInstruction(HEIGHT_CAN_CHANGE);
        addSubview(textView);

        /* cover the text with a ScreenView (grid of lines) */
        screenView = new ScreenView(interiorRect.x, interiorRect.y,
                                    interiorRect.width, interiorRect.height);
        screenView.setHorizResizeInstruction(WIDTH_CAN_CHANGE);
        screenView.setVertResizeInstruction(HEIGHT_CAN_CHANGE);
        addSubview(screenView);
        if (file != null) {
            loadFile(file);               // Try to load file if specified
        }
    }

    // --------------  Animation control methods  --------------

    /** Set the textView to the first string and start the ScreenView
      * animation.
      */
    public void start() {
        index = 0;
        screenView.start();
        textView.setStringValue((String)(strings.elementAt(index)));
        textView.moveTo(textView.bounds.x,
                         (bounds.height - textView.bounds.height) / 2);
        super.start();
    }

    /** Stop the ScreenView animation and redraw. */
    public void stop() {
        super.stop();

        screenView.stop();
        textView.setStringValue(null);
        draw();
    }

    /** Pause the ScreenView animation. */
    public void pause() {
        super.pause();

        screenView.pause();
    }

    /** Set the speed of the ScreenView animation. */
    public void setSpeed(float speedFactor) {
        super.setSpeed(speedFactor);

        screenView.setSpeed(speedFactor);
    }

    /** Sent by the ScreenView when it's time to change strings. */
    public void advance() {
        if (++index == strings.count()) {
            index = 0;
        }

        textView.setStringValue((String)(strings.elementAt(index)));

        /* re-center the textView */
        textView.moveTo(textView.bounds.x,
                             (bounds.height - textView.bounds.height) / 2);
    }

    /** Overridden to convert everything to upper case. */
    public void loadFile(String file) {
        int     i, n;

        super.loadFile(file);
        n = strings.count();
        for (i = 0; i < n; i++) {
            strings.setElementAt(
                ((String)(strings.elementAt(i))).toUpperCase(), i);
        }
    }
}
