// SkyImage.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** A DrawingSequence that draws a couple of cloud images over a sky-blue
  * background.
  */
public class SkyImage extends DrawingSequence {
    Color  backgroundColor;  // The background color
    Bitmap clouds[];         // The cloud bitmaps
    Rect   cloudRects[];     // The current cloud rectangles

    // --------------  Constructors and initialization  --------------

    public SkyImage() {
        backgroundColor = new Color(128, 192, 255);
        clouds = new Bitmap[2];
        cloudRects = new Rect[2];

        /* load the cloud Bitmaps */
        clouds[0] = Bitmap.bitmapNamed("cloud1.gif");
        clouds[1] = Bitmap.bitmapNamed("cloud2.gif");
        cloudRects[0] = new Rect(0, 0, clouds[0].width(), clouds[0].height());
        cloudRects[1] = new Rect(150, 0, clouds[1].width(),
                                 clouds[1].height());
        setPlaybackMode(FORWARD_LOOP);
        setFrameCount(200);  // Arbitrary; we don't use the frame count
        setFrameRate(500);   // Update twice per second
    }

    public Color backgroundColor() {
        return backgroundColor;
    }

    // -----------------  DrawingSequence methods  ----------------------

    public void drawScaled(Graphics g, int x, int y, int width, int height) {
        int     i, spacing;

        spacing = height / (clouds.length + 2);

        /* first paint the background color into the clipping rectangle */
        g.setColor(backgroundColor);
        g.fillRect(g.clipRect());

        /* now draw each cloud, spreading them out vertically */
        for (i = 0; i < clouds.length; i++) {
            if ((i % 2) == 0) {
                cloudRects[i].y = y + spacing * ((i / 2) + 1) -
                                                    cloudRects[i].height / 2;
            } else {
                cloudRects[i].y = y + height - spacing * ((i / 2) + 1) -
                                                    cloudRects[i].height / 2;
            }
            if (cloudRects[i].x > width) {
                cloudRects[i].x = x - cloudRects[i].width;
            }
            if (g.clipRect().intersects(cloudRects[i])) {
                g.drawBitmapAt(clouds[i], cloudRects[i].x, cloudRects[i].y);
            }
        }
    }

    public void drawAt(Graphics g, int x, int y) {
        drawScaled(g, x, y, 100, 100);
    }

    /** Increment the cloud rectangle x positions by one pixel.
      */
    public boolean nextFrame() {
        int     i;

        for (i = 0; i < cloudRects.length; i++) {
            cloudRects[i].x++;
        }

        return super.nextFrame();
    }

    public int height() {
        return cloudRects[0].height;
    }

    public int width() {
        return cloudRects[0].width;
    }
}
