// SpaceImage.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.


import netscape.application.*;
import netscape.util.*;

/** A DrawingSequence which animates (by default) an image of stars and
  * galaxies, tiling the image and moving it slowly to the right and down.
  */
public class SpaceImage extends DrawingSequence {
    Bitmap bitmap;
    int    xoffset;
    int    yoffset;
    int    bitmapWidth;
    int    bitmapHeight;

    // --------------  Constructors and initialization  --------------

    public SpaceImage() {
        bitmap = Bitmap.bitmapNamed("galaxies.gif");
        setPlaybackMode(FORWARD_LOOP);
        setFrameCount(10);
        setFrameRate(70);
        xoffset = 0;
        yoffset = 0;
        bitmapWidth = bitmap.width();
        bitmapHeight = bitmap.height();
    }


    // -----------------  DrawingSequence methods  ----------------------

    /** Tile the Image, starting at the current x and y offset.
      */
    public void drawTiled(Graphics g, int x, int y, int width, int height) {
        int     startx, starty, lastx, lasty;

        g.pushState();
        g.setClipRect(new Rect(x, y, width, height));

        /* if the offsets are less than the width or height of the Bitmap,
         * start the tiling to the left and/or above the drawing area
         */
        startx = x + ((xoffset == 0) ? 0 : (xoffset - bitmapWidth));
        starty = y + ((yoffset == 0) ? 0 : (yoffset - bitmapHeight));

        lastx = x + width;  // Stop tiling here
        lasty = y + height;

        x = startx;
        while (x < lastx) {
            y = starty;
            while (y < lasty) {
                if (g.clipRect().intersects(
                                 new Rect(x, y, bitmapWidth, bitmapHeight))) {
                    g.drawBitmapAt(bitmap, x, y);
                }
                y += bitmapHeight;
            }
            x += bitmapWidth;
        }

        g.popState();
    }

    public void drawAt(Graphics g, int x, int y) {
        drawScaled(g, x, y, bitmapWidth, bitmapHeight);
    }

    public void drawScaled(Graphics g, int x, int y, int width, int height) {
        drawTiled(g, x, y, width, height);
    }

    /** Called by the DrawingSequence's Timer to set the current frame. We
      * don't care about that here, we just increment the offsets.
      */
    public boolean nextFrame() {
        xoffset++;
        yoffset++;
        if (xoffset >= bitmapWidth) {
            xoffset = 0;
        }
        if (yoffset >= bitmapHeight) {
            yoffset = 0;
        }
        return super.nextFrame();
    }

    public int height() {
        return bitmapHeight;
    }

    public int width() {
        return bitmapWidth;
    }
}
