// TunnelImage.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.


import netscape.application.*;
import netscape.util.*;

/** Draws gradually darkening concentric rectangles, simulating a kind
  * of tunnel effect.
  */
public class TunnelImage extends Image {
    int width, height;

    public TunnelImage(int width, int height) {
        this.width = width;
        this.height = height;
    }

    public TunnelImage() {
        this(5, 5);
    }

    public void drawScaled(Graphics g, int x, int y,
                                    int width, int height) {
        Rect  rect;               // The drawn rectangle
        int   gray;               // Current gray shade
        float interp;             // Used for interpolating the grays
        float increment;          // Gray adjustment increment per rectangle

        rect = new Rect(x, y, width, height);
        interp = 1.0f;
        increment = 1.0f / (float)(Math.max(width, height) / 2);

        while ((rect.width > 0) || (rect.height > 0)) {
            gray = (int)(interp * 255.0f); // compute the current gray level
            interp -= increment;
            g.setColor(new Color(gray, gray, gray)); // reset the color
            g.drawRect(rect);
            rect.growBy(-1, -1);          // shrink the rectangle
        }

    }

    public void drawAt(Graphics g, int x, int  y) {
        drawScaled(g, x, y, width, height);
    }

    public int width() {
        return width;
    }

    public int height() {
        return height;
    }
}
